# Commerce Recurring Donation Widget

This Drupal module provides a flexible, configurable donation widget for Drupal Commerce, supporting both one-time and recurring (monthly/yearly) donations. It integrates with Commerce Recurring and allows for custom donation metadata, including dedications, designations, and comments.

## Features

- **Flexible Configuration**: Two-tier system with global defaults and per-block overrides
- **Multiple Donation Types**: One-time, monthly, and yearly donations with customizable labels
- **Product Variation Integration**: Each donation option linked to a Commerce Product Variation
- **Custom Amounts**: Allow donors to enter custom donation amounts
- **Recurring Donations**: Full integration with Commerce Recurring module (optional)
- **Dedication Support**: "In honor of" / "in memory of" dedications with names and messages
- **Designation System**: Configurable fund/purpose selection for donations
- **Donor Comments**: Optional comment field for donations
- **AJAX Cart Editing**: Edit dedications, designations, and comments directly in cart
- **Auto-Configuration**: Automatically sets up cart view modes for donation variations
- **Mobile Responsive**: Modern, mobile-friendly styling
- **Multi-Block Support**: Different widgets on different pages with varying features
- **Full Accessibility**: Complete keyboard navigation and screen reader support
- **Drupal 9/10/11 Compatible**

## Accessibility Features

This module is fully accessible and follows WCAG 2.1 AA guidelines:

### Keyboard Navigation

**Donation Type Selector (Tabs)**
- Use `Tab` to focus the donation type selector
- Use `Arrow Keys` (Left/Right/Up/Down) to navigate between donation types
- Use `Home` to jump to the first option, `End` to jump to the last
- Use `Enter` or `Space` to select a donation type

**Donation Amount Options (Radio Group)**
- Use `Tab` to focus the donation amounts section
- Use `Arrow Keys` (Left/Right/Up/Down) to navigate between amount options
- Use `Home` to jump to the first option, `End` to jump to the last
- Use `Enter` or `Space` to select an amount

**Form Fields**
- All form fields are properly labeled and focusable
- Use `Tab` and `Shift+Tab` to navigate between fields
- Required fields are clearly marked with `*` and `aria-required="true"`

### Screen Reader Support

- **ARIA Labels**: All interactive elements have descriptive ARIA labels
- **Live Regions**: Status updates are announced to screen readers
- **Semantic HTML**: Proper use of headings, fieldsets, and form elements
- **Hidden Content**: Screen reader-only instructions and descriptions
- **State Changes**: Selection changes and form updates are announced

### Visual Accessibility

- **Focus Indicators**: Clear, high-contrast focus outlines on all interactive elements
- **High Contrast Mode**: Enhanced outlines for high contrast display preferences
- **Reduced Motion**: Respects `prefers-reduced-motion` for users with vestibular disorders
- **Color Independence**: Information is not conveyed through color alone

### Keyboard Shortcuts Summary

| Element | Keys | Action |
|---------|------|--------|
| Donation Types | `Arrow Keys` | Navigate between types |
| Donation Types | `Enter`/`Space` | Select type |
| Donation Types | `Home`/`End` | Jump to first/last |
| Amount Options | `Arrow Keys` | Navigate between amounts |
| Amount Options | `Enter`/`Space` | Select amount |
| Amount Options | `Home`/`End` | Jump to first/last |
| All Elements | `Tab`/`Shift+Tab` | Navigate forward/backward |

### Screen Reader Announcements

The widget provides live announcements for:
- Donation type changes ("Switched to Monthly donations")
- Amount selection ("Amount $25 selected")
- Custom amount activation ("Custom amount option selected. Please enter your amount.")
- Form validation errors with focus management
- Submission status ("Submitting your donation...", "Donation submitted successfully")

## Requirements

- Drupal 9.x, 10.x, or 11.x
- Commerce 2.x
- Commerce Recurring module (optional, for recurring donations)

## Installation

1. **Install via Composer** (if packaged):
   ```sh
   composer require drupal/commerce_recurring_donation_widget
   ```
   
2. **Enable the module**:
   ```sh
   drush en commerce_recurring_donation_widget
   ```

3. **Optional: Install Commerce Recurring** (for monthly/yearly donations):
   ```sh
   composer require drupal/commerce_recurring
   drush en commerce_recurring
   ```

4. **Automatic Setup**: The module automatically:
   - Creates donation order and order item types
   - Adds donation metadata fields
   - Configures cart view modes for donation product variations
   - Sets up proper field attachments

**Installation Order**: Commerce Recurring can be installed before or after this module - the auto-configuration handles both scenarios.

## Configuration

The module uses a two-tier configuration system: **Global Settings** that apply site-wide, and **Block Settings** that can override globals per block instance.

### 1. Global Settings

Configure site-wide defaults at **Commerce > Configuration > Donation Widget**:

**Donation Options**
- Add donation options linked to Commerce Product Variations
- Each option includes: Title, Product Variation, Donation Type (one-time/monthly/yearly)
- Mark options as "Custom Amount" to allow user-entered amounts
- Options are grouped by type in the admin interface for easy management

**Display Settings** (Global Defaults)
- Enable Dedication: Allow "in honor of" / "in memory of" dedications
- Enable Comments: Allow donation comments
- Enable Designation: Allow fund/purpose selection
- Enable Custom Amount: Global toggle for custom amount entry

**Donation Type Labels**
- Customize labels for each donation type (one-time, monthly, yearly)
- Used in both admin interface and donor-facing forms

**Designations**
- Create designation options with ID and label (e.g., "general_fund" → "General Fund")
- Set a default designation (optional)

### 2. Block Configuration

Each **Recurring Donation Widget** block can override global settings:

**Content Filtering**
- **Donation Options**: Select which global options to show (filters global list)
- **Designations**: Select which designations to show (filters global list)

**Display Overrides** (Override Global Settings)
- **Enable Dedication**: Override global dedication setting for this block
- **Enable Comments**: Override global comment setting for this block  
- **Enable Designation**: Override global designation setting for this block
- **Default Designation**: Set block-specific default (overrides global)
- **Custom Amount Placeholder**: Customize placeholder text for this block

**How It Works**: Block settings take precedence over global settings. If a block doesn't specify a setting, it inherits from global configuration. This allows you to have different donation widgets on different pages with varying features.

## Order Item Fields

The following fields are added to each donation order item:

- `field_dedication_type` (string): Dedication type ("honor" or "memory")
- `field_dedication_name` (string): Name of the person being honored or remembered
- `field_dedication_message` (text): Dedication message
- `field_designation` (string): Designation/fund ID
- `field_donation_comment` (text): Donor comment

These fields are editable by the donor during checkout and via AJAX modals in the cart.

## Recurring Billing Schedules

For recurring donations, ensure you have billing schedules set up in Commerce Recurring:

- One with the plugin ID `monthly` for monthly donations
- One with the plugin ID `yearly` for yearly donations

## Customization

- **Styling**: Override the provided CSS in your theme for custom appearance.
- **Templates**: Override the Twig templates in your theme for custom markup.

## Troubleshooting

### Common Issues

**Empty donation titles in cart**
- Run `drush updatedb` to apply the latest cart view mode configurations
- The module automatically configures cart display for donation product variations

**Recurring donations not working**
- Install Commerce Recurring module: `composer require drupal/commerce_recurring`
- Ensure billing schedules exist with plugin IDs: `monthly`, `yearly`
- Check that product variations are published and properly configured

**Missing donation fields in cart/checkout**
- Clear all caches: `drush cr`
- Run database updates: `drush updatedb`  
- Check that order items are using the 'donation' order item type

**Block configuration not taking effect**
- Remember: Block settings override global settings
- Check that the block has the correct configuration
- Verify global settings are properly configured as fallbacks

**Custom product variation types**
- The module automatically configures cart view modes for any donation-related variation types
- No manual configuration needed for custom types like 'recurring_donation'

**Accessibility Issues**
- Ensure JavaScript is enabled for full keyboard navigation
- Test with screen readers like NVDA, JAWS, or VoiceOver
- Check that focus indicators are visible in your theme
- Verify that custom CSS doesn't override accessibility styles

## License

This project is licensed under the GPL v2 or later.

## Cart Display & AJAX Editing

### Automatic Integration

The module automatically:
- Adds donation fields to order items in the cart
- Provides AJAX edit links for dedications, designations, and comments
- Configures proper cart view modes for donation product variations

### Template Override (Optional)

For full control over cart display, copy the example template:

**From**: `web/modules/custom/commerce_recurring_donation_widget/templates/examples/`  
**To**: `your-theme/templates/`

**Available templates**:
- `views-view-table--commerce-cart-form.html.twig` - Full cart editing
- `views-view-table--commerce-checkout-order-summary.html.twig` - Checkout summary

### Template Features

The example templates provide:
- AJAX modal editing for all donation fields
- Proper data attributes for JavaScript functionality  
- Responsive design and accessibility
- Add/Edit/Remove functionality for dedications, designations, comments

### AJAX Editing URLs

The module provides these AJAX endpoints:
- `/commerce-recurring-donation-widget/dedication/edit/{order_item_id}/{field}`
- `/commerce-recurring-donation-widget/designation/edit/{order_item_id}`
- `/commerce-recurring-donation-widget/comment/edit/{order_item_id}`

### Customization

- Override templates in your theme for custom styling
- Use `hook_preprocess_views_view_table()` for advanced customization
- All AJAX forms are fully customizable via standard Drupal form alter hooks 

## Testing Accessibility

### Manual Testing

1. **Keyboard Navigation**:
   - Disconnect your mouse and navigate using only the keyboard
   - Ensure all functionality is accessible via keyboard
   - Check that focus indicators are clearly visible

2. **Screen Reader Testing**:
   - Test with NVDA (Windows), VoiceOver (macOS), or Orca (Linux)
   - Verify that all content is announced properly
   - Check that form validation errors are announced

3. **High Contrast Mode**:
   - Enable high contrast mode in your OS
   - Verify that focus indicators remain visible
   - Check that all text has sufficient contrast

### Automated Testing

Consider using these tools for accessibility testing:
- **axe-core**: Browser extension for automated accessibility testing
- **WAVE**: Web accessibility evaluation tool
- **Lighthouse**: Built-in Chrome accessibility audit
- **Pa11y**: Command-line accessibility testing tool
